@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2004-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -89,6 +89,14 @@ public:
 
     void BuildAttrs()
     {
+        for( wxGridCellAttr* attr : m_nameAttrs )
+            attr->DecRef();
+
+        m_nameAttrs.clear();
+
+        if( m_readOnlyAttr )
+            m_readOnlyAttr->DecRef();
+
         m_readOnlyAttr = new wxGridCellAttr;
         m_readOnlyAttr->SetReadOnly( true );
 
@@ -117,11 +125,17 @@ public:
             m_nameAttrs.push_back( attr );
         }
 
+        if( m_typeAttr )
+            m_typeAttr->DecRef();
+
         m_typeAttr = new wxGridCellAttr;
         m_typeAttr->SetRenderer( new GRID_CELL_ICON_TEXT_RENDERER( PinTypeIcons(),
                                                                    PinTypeNames() ) );
         m_typeAttr->SetReadOnly( true );
 
+        if( m_shapeAttr )
+            m_shapeAttr->DecRef();
+
         m_shapeAttr = new wxGridCellAttr;
         m_shapeAttr->SetRenderer( new GRID_CELL_ICON_TEXT_RENDERER( PinShapeIcons(),
                                                                     PinShapeNames() ) );
@@ -635,7 +649,7 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataFromWindow()
 
     switch( m_mirrorCtrl->GetSelection() )
     {
-    case 0:                                         break;
+    case 0:                                           break;
     case 1: m_symbol->SetOrientation( SYM_MIRROR_X ); break;
     case 2: m_symbol->SetOrientation( SYM_MIRROR_Y ); break;
     }
@@ -1024,6 +1038,7 @@ void DIALOG_SYMBOL_PROPERTIES::OnPinTableColSort( wxGridEvent& aEvent )
         ascending = true;
 
     m_dataModel->SortRows( sortCol, ascending );
+    m_dataModel->BuildAttrs();
 }
 
 
@@ -1139,3 +1154,16 @@ void DIALOG_SYMBOL_PROPERTIES::OnChoice( wxCommandEvent& event )
 {
     OnModify();
 }
+
+
+void DIALOG_SYMBOL_PROPERTIES::onUpdateEditSymbol( wxUpdateUIEvent& event )
+{
+    event.Enable( m_symbol && m_symbol->GetLibSymbolRef() );
+}
+
+
+void DIALOG_SYMBOL_PROPERTIES::onUpdateEditLibrarySymbol( wxUpdateUIEvent& event )
+{
+    event.Enable( m_symbol && m_symbol->GetLibSymbolRef() );
+}
+
