@@ -456,6 +456,7 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
             // Legacy schematic can have duplicate time stamps so fix that before converting
             // to the s-expression format.
             schematic.ReplaceDuplicateTimeStamps();
+            schematic.SetLegacySymbolInstanceData();
 
             // Allow the schematic to be saved to new file format without making any edits.
             OnModify();
@@ -516,6 +517,8 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
     SyncView();
     GetScreen()->ClearDrawingState();
 
+    TestDanglingEnds();
+
     UpdateHierarchyNavigator();
     UpdateTitle();
 
@@ -632,6 +635,10 @@ void SCH_EDIT_FRAME::OnImportProject( wxCommandEvent& aEvent )
     if( dlg.ShowModal() == wxID_CANCEL )
         return;
 
+    // Don't leave dangling pointers to previously-opened document.
+    m_toolManager->GetTool<EE_SELECTION_TOOL>()->ClearSelection();
+    ClearUndoRedoList();
+
     if( setProject )
     {
         Schematic().SetProject( nullptr );
@@ -665,8 +672,6 @@ void SCH_EDIT_FRAME::OnImportProject( wxCommandEvent& aEvent )
         return;
     }
 
-    m_toolManager->GetTool<EE_SELECTION_TOOL>()->ClearSelection();
-
     importFile( dlg.GetPath(), pluginType );
 
     RefreshCanvas();
