@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -95,21 +95,22 @@ static LIB_SYMBOL* dummy()
 }
 
 
-SCH_SYMBOL::SCH_SYMBOL( const wxPoint& aPos, SCH_ITEM* aParent ) :
+SCH_SYMBOL::SCH_SYMBOL( const wxPoint& aPosition, EDA_ITEM* aParent ) :
     SCH_ITEM( aParent, SCH_SYMBOL_T )
 {
-    Init( aPos );
+    Init( aPosition );
 }
 
 
 SCH_SYMBOL::SCH_SYMBOL( const LIB_SYMBOL& aSymbol, const LIB_ID& aLibId,
-                        const SCH_SHEET_PATH* aSheet, int unit, int convert, const wxPoint& pos ) :
-    SCH_ITEM( nullptr, SCH_SYMBOL_T )
+                        const SCH_SHEET_PATH* aSheet, int aUnit, int aConvert,
+                        const wxPoint& aPosition, EDA_ITEM* aParent ) :
+    SCH_ITEM( aParent, SCH_SYMBOL_T )
 {
-    Init( pos );
+    Init( aPosition );
 
-    m_unit      = unit;
-    m_convert   = convert;
+    m_unit      = aUnit;
+    m_convert   = aConvert;
     m_lib_id    = aLibId;
 
     std::unique_ptr< LIB_SYMBOL > part;
@@ -146,8 +147,8 @@ SCH_SYMBOL::SCH_SYMBOL( const LIB_SYMBOL& aSymbol, const LIB_ID& aLibId,
 
 
 SCH_SYMBOL::SCH_SYMBOL( const LIB_SYMBOL& aSymbol, const SCH_SHEET_PATH* aSheet,
-                        const PICKED_SYMBOL& aSel, const wxPoint& pos ) :
-    SCH_SYMBOL( aSymbol, aSel.LibId, aSheet, aSel.Unit, aSel.Convert, pos )
+                        const PICKED_SYMBOL& aSel, const wxPoint& aPosition, EDA_ITEM* aParent ) :
+    SCH_SYMBOL( aSymbol, aSel.LibId, aSheet, aSel.Unit, aSel.Convert, aPosition, aParent )
 {
     // Set any fields that were modified as part of the symbol selection
     for( const std::pair<int, wxString>& i : aSel.Fields )
@@ -226,6 +227,15 @@ EDA_ITEM* SCH_SYMBOL::Clone() const
 }
 
 
+bool SCH_SYMBOL::IsMissingLibSymbol() const
+{
+    if( !m_part )
+        return true;
+
+    return false;
+}
+
+
 void SCH_SYMBOL::ViewGetLayers( int aLayers[], int& aCount ) const
 {
     aCount      = 4;
@@ -237,6 +247,27 @@ void SCH_SYMBOL::ViewGetLayers( int aLayers[], int& aCount ) const
 }
 
 
+bool SCH_SYMBOL::IsMovableFromAnchorPoint() const
+{
+    // If a symbol's anchor is not grid-aligned to its pins then moving from the anchor is
+    // going to end up moving the symbol's pins off-grid.
+
+    // The minimal grid size allowed to place a pin is 25 mils
+    const int min_grid_size = Mils2iu( 25 );
+
+    for( const std::unique_ptr<SCH_PIN>& pin : m_pins )
+    {
+        if( ( ( pin->GetPosition().x - m_pos.x ) % min_grid_size ) != 0 )
+            return false;
+
+        if( ( ( pin->GetPosition().y - m_pos.y ) % min_grid_size ) != 0 )
+            return false;
+    }
+
+    return true;
+}
+
+
 void SCH_SYMBOL::SetLibId( const LIB_ID& aLibId )
 {
     if( m_lib_id != aLibId )
@@ -483,27 +514,29 @@ bool SCH_SYMBOL::IsReferenceStringValid( const wxString& aReferenceString )
 void SCH_SYMBOL::SetRef( const SCH_SHEET_PATH* sheet, const wxString& ref )
 {
     KIID_PATH path = sheet->Path();
-    bool      notInArray = true;
+    bool      found = false;
 
     // check to see if it is already there before inserting it
     for( SYMBOL_INSTANCE_REFERENCE& instance : m_instanceReferences )
     {
         if( instance.m_Path == path )
         {
+            found = true;
             instance.m_Reference = ref;
-            notInArray = false;
+            break;
         }
     }
 
-    if( notInArray )
-        AddHierarchicalReference( path, ref, m_unit );
+    if( !found )
+        AddHierarchicalReference( path, ref, m_unit, GetField( VALUE_FIELD )->GetText(),
+                                  GetField( FOOTPRINT_FIELD )->GetText() );
 
     for( std::unique_ptr<SCH_PIN>& pin : m_pins )
         pin->ClearDefaultNetName( sheet );
 
-    SCH_FIELD* rf = GetField( REFERENCE_FIELD );
-
-    rf->SetText( ref );  // for drawing.
+    // Schematic() is null for new components just added
+    if( !Schematic() || *sheet == Schematic()->CurrentSheet() )
+        m_fields[ REFERENCE_FIELD ].SetText( ref );
 
     // Reinit the m_prefix member if needed
     m_prefix = UTIL::GetRefDesPrefix( ref );
@@ -604,29 +637,37 @@ void SCH_SYMBOL::SetValue( const SCH_SHEET_PATH* sheet, const wxString& aValue )
 {
     if( sheet == nullptr )
     {
-        // Clear instance overrides and set primary field value
+        // Set all instances to the updated value
         for( SYMBOL_INSTANCE_REFERENCE& instance : m_instanceReferences )
-            instance.m_Value = wxEmptyString;
+            instance.m_Value = aValue;
 
         m_fields[ VALUE_FIELD ].SetText( aValue );
         return;
     }
 
     KIID_PATH path = sheet->Path();
+    bool      found = false;
 
     // check to see if it is already there before inserting it
     for( SYMBOL_INSTANCE_REFERENCE& instance : m_instanceReferences )
     {
         if( instance.m_Path == path )
         {
+            found = true;
             instance.m_Value = aValue;
-            return;
+            break;
         }
     }
 
     // didn't find it; better add it
-    AddHierarchicalReference( path, UTIL::GetRefDesUnannotated( m_prefix ), m_unit,
-                              aValue, wxEmptyString );
+    if( !found )
+    {
+        AddHierarchicalReference( path, UTIL::GetRefDesUnannotated( m_prefix ), m_unit, aValue,
+                                  wxEmptyString );
+    }
+
+    if( Schematic() && *sheet == Schematic()->CurrentSheet() )
+        m_fields[ VALUE_FIELD ].SetText( aValue );
 }
 
 
@@ -655,29 +696,37 @@ void SCH_SYMBOL::SetFootprint( const SCH_SHEET_PATH* sheet, const wxString& aFoo
 {
     if( sheet == nullptr )
     {
-        // Clear instance overrides and set primary field value
+        // Set all instances to new footprint value
         for( SYMBOL_INSTANCE_REFERENCE& instance : m_instanceReferences )
-            instance.m_Footprint = wxEmptyString;
+            instance.m_Footprint = aFootprint;
 
         m_fields[ FOOTPRINT_FIELD ].SetText( aFootprint );
         return;
     }
 
     KIID_PATH path = sheet->Path();
+    bool      found = false;
 
     // check to see if it is already there before inserting it
     for( SYMBOL_INSTANCE_REFERENCE& instance : m_instanceReferences )
     {
         if( instance.m_Path == path )
         {
+            found = true;
             instance.m_Footprint = aFootprint;
-            return;
+            break;
         }
     }
 
     // didn't find it; better add it
-    AddHierarchicalReference( path, UTIL::GetRefDesUnannotated( m_prefix ), m_unit,
-                              wxEmptyString, aFootprint );
+    if( !found )
+    {
+        AddHierarchicalReference( path, UTIL::GetRefDesUnannotated( m_prefix ), m_unit,
+                                  wxEmptyString, aFootprint );
+    }
+
+    if( Schematic() && *sheet == Schematic()->CurrentSheet() )
+        m_fields[ FOOTPRINT_FIELD ].SetText( aFootprint );
 }
 
 
@@ -865,7 +914,7 @@ void SCH_SYMBOL::GetLibPins( std::vector<LIB_PIN*>& aPinsList ) const
 }
 
 
-SCH_PIN* SCH_SYMBOL::GetPin( LIB_PIN* aLibPin )
+SCH_PIN* SCH_SYMBOL::GetPin( LIB_PIN* aLibPin ) const
 {
     wxASSERT( m_pinMap.count( aLibPin ) );
     return m_pins[ m_pinMap.at( aLibPin ) ].get();
@@ -1856,10 +1905,32 @@ void SCH_SYMBOL::Plot( PLOTTER* aPlotter ) const
 {
     if( m_part )
     {
+        LIB_PINS  libPins;
+        m_part->GetPins( libPins, GetUnit(), GetConvert() );
+
+        // Copy the source so we can re-orient and translate it.
+        LIB_SYMBOL tempSymbol( *m_part );
+        LIB_PINS tempPins;
+        tempSymbol.GetPins( tempPins, GetUnit(), GetConvert() );
+
+        // Copy the pin info from the symbol to the temp pins
+        for( unsigned i = 0; i < tempPins.size(); ++ i )
+        {
+            SCH_PIN* symbolPin = GetPin( libPins[ i ] );
+            LIB_PIN* tempPin = tempPins[ i ];
+
+            tempPin->SetName( symbolPin->GetShownName() );
+            tempPin->SetType( symbolPin->GetType() );
+            tempPin->SetShape( symbolPin->GetShape() );
+
+            if( symbolPin->IsDangling() )
+                tempPin->SetFlags( IS_DANGLING );
+        }
+
         TRANSFORM temp = GetTransform();
         aPlotter->StartBlock( nullptr );
 
-        m_part->Plot( aPlotter, GetUnit(), GetConvert(), m_pos, temp );
+        tempSymbol.Plot( aPlotter, GetUnit(), GetConvert(), m_pos, temp );
 
         for( SCH_FIELD field : m_fields )
             field.Plot( aPlotter );
@@ -1869,6 +1940,35 @@ void SCH_SYMBOL::Plot( PLOTTER* aPlotter ) const
 }
 
 
+void SCH_SYMBOL::PlotPins( PLOTTER* aPlotter ) const
+{
+    if( m_part )
+    {
+        LIB_PINS  libPins;
+        m_part->GetPins( libPins, GetUnit(), GetConvert() );
+
+        // Copy the source to stay const
+        LIB_SYMBOL tempSymbol( *m_part );
+        LIB_PINS tempPins;
+        tempSymbol.GetPins( tempPins, GetUnit(), GetConvert() );
+
+        TRANSFORM transform = GetTransform();
+
+        // Copy the pin info from the symbol to the temp pins
+        for( unsigned i = 0; i < tempPins.size(); ++ i )
+        {
+            SCH_PIN* symbolPin = GetPin( libPins[ i ] );
+            LIB_PIN* tempPin = tempPins[ i ];
+
+            tempPin->SetName( symbolPin->GetShownName() );
+            tempPin->SetType( symbolPin->GetType() );
+            tempPin->SetShape( symbolPin->GetShape() );
+            tempPin->Plot( aPlotter, m_pos, false, transform);
+        }
+    }
+}
+
+
 bool SCH_SYMBOL::HasBrightenedPins()
 {
     for( const auto& pin : m_pins )
