@@ -31,6 +31,7 @@
 #include <pgm_base.h>
 #include <kiway.h>
 #include <profile.h>
+#include <wx_filename.h>
 #include <sch_io_mgr.h>
 #include <sch_plugins/legacy/sch_legacy_plugin.h>
 #include <symbol_lib_table.h>
@@ -210,10 +211,10 @@ bool SYMBOL_LIBRARY_MANAGER::SaveLibrary( const wxString& aLibrary, const wxStri
         if( row )
         {
             original = row->GetFullURI( true );
-            original.Normalize();
+            original.Normalize( FN_NORMALIZE_FLAGS | wxPATH_NORM_ENV_VARS );
         }
 
-        destination.Normalize();
+        destination.Normalize( FN_NORMALIZE_FLAGS | wxPATH_NORM_ENV_VARS );
 
         if( res && original == destination )
             libBuf.ClearDeletedBuffer();
@@ -225,30 +226,38 @@ bool SYMBOL_LIBRARY_MANAGER::SaveLibrary( const wxString& aLibrary, const wxStri
         {
             LIB_SYMBOL* newSymbol;
 
-            if( symbol->IsAlias() )
+            try
             {
-                std::shared_ptr< LIB_SYMBOL > oldParent = symbol->GetParent().lock();
-
-                wxCHECK_MSG( oldParent, false,
-                             wxString::Format( wxT( "Derived symbol '%s' found with undefined parent." ),
-                                               symbol->GetName() ) );
-
-                LIB_SYMBOL* libParent = pi->LoadSymbol( aLibrary, oldParent->GetName(),
-                                                        &properties );
-
-                if( !libParent )
+                if( symbol->IsAlias() )
                 {
-                    libParent = new LIB_SYMBOL( *oldParent.get() );
-                    pi->SaveSymbol( aLibrary, libParent, &properties );
-                }
+                    std::shared_ptr< LIB_SYMBOL > oldParent = symbol->GetParent().lock();
 
-                newSymbol = new LIB_SYMBOL( *symbol );
-                newSymbol->SetParent( libParent );
-                pi->SaveSymbol( aLibrary, newSymbol, &properties );
+                    wxCHECK_MSG( oldParent, false,
+                                 wxString::Format( wxT( "Derived symbol '%s' found with undefined parent." ),
+                                                   symbol->GetName() ) );
+
+                    LIB_SYMBOL* libParent = pi->LoadSymbol( aLibrary, oldParent->GetName(),
+                                                            &properties );
+
+                    if( !libParent )
+                    {
+                        libParent = new LIB_SYMBOL( *oldParent.get() );
+                        pi->SaveSymbol( aLibrary, libParent, &properties );
+                    }
+
+                    newSymbol = new LIB_SYMBOL( *symbol );
+                    newSymbol->SetParent( libParent );
+                    pi->SaveSymbol( aLibrary, newSymbol, &properties );
+                }
+                else if( !pi->LoadSymbol( aLibrary, symbol->GetName(), &properties ) )
+                {
+                    pi->SaveSymbol( aLibrary, new LIB_SYMBOL( *symbol ), &properties );
+                }
             }
-            else if( !pi->LoadSymbol( aLibrary, symbol->GetName(), &properties ) )
+            catch( ... )
             {
-                pi->SaveSymbol( aLibrary, new LIB_SYMBOL( *symbol ), &properties );
+                res = false;
+                break;
             }
         }
     }
@@ -951,11 +960,23 @@ bool SYMBOL_LIBRARY_MANAGER::LIB_BUFFER::SaveBuffer( std::shared_ptr<SYMBOL_BUFF
     PROPERTIES properties;
     properties.emplace( SCH_LEGACY_PLUGIN::PropBuffering, wxEmptyString );
 
+    wxString errorMsg = _( "Error saving symbol %s to library '%s'." ) + wxS( "\n%s" );
+
     // Delete the original symbol if the symbol name has been changed.
     if( libSymbol->GetName() != originalSymbol->GetName() )
     {
-        if( aLibTable->LoadSymbol( m_libName, originalSymbol->GetName() ) )
-            aLibTable->DeleteSymbol( m_libName, originalSymbol->GetName() );
+        // DeleteSymbol may throw
+        try
+        {
+            if( aLibTable->LoadSymbol( m_libName, originalSymbol->GetName() ) )
+                aLibTable->DeleteSymbol( m_libName, originalSymbol->GetName() );
+        }
+        catch( const IO_ERROR& ioe )
+        {
+            wxLogError( errorMsg, UnescapeString( originalSymbol->GetName() ), m_libName,
+                        ioe.What() );
+            return false;
+        }
     }
 
     if( libSymbol->IsAlias() )
@@ -1046,8 +1067,17 @@ bool SYMBOL_LIBRARY_MANAGER::LIB_BUFFER::SaveBuffer( std::shared_ptr<SYMBOL_BUFF
     // Delete the original symbol if the symbol name has been changed.
     if( libSymbol->GetName() != originalSymbol->GetName() )
     {
-        if( aPlugin->LoadSymbol( aFileName, originalSymbol->GetName() ) )
-            aPlugin->DeleteSymbol( aFileName, originalSymbol->GetName(), &properties );
+        try
+        {
+            if( aPlugin->LoadSymbol( aFileName, originalSymbol->GetName() ) )
+                aPlugin->DeleteSymbol( aFileName, originalSymbol->GetName(), &properties );
+        }
+        catch( const IO_ERROR& ioe )
+        {
+            wxLogError( errorMsg, UnescapeString( originalSymbol->GetName() ), aFileName,
+                        ioe.What() );
+            return false;
+        }
     }
 
     if( libSymbol->IsAlias() )
