@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2019 CERN
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -62,6 +62,7 @@
 #include <eda_list_dialog.h>
 
 #include <wildcards_and_files_ext.h>
+#include <wx_filename.h>
 #include <sch_sheet_path.h>
 #include <wx/filedlg.h>
 
@@ -499,19 +500,12 @@ int SCH_EDITOR_CONTROL::FindNext( const TOOL_EVENT& aEvent )
                 continue;
             }
 
-            SCH_SCREEN* screen = sheet->LastScreen();
-
-            item = nextMatch( screen, sheet, nullptr, data );
+            item = nextMatch( sheet->LastScreen(), sheet, nullptr, data );
 
             if( item )
             {
                 m_frame->Schematic().SetCurrentSheet( *sheet );
-                m_frame->GetCurrentSheet().UpdateAllScreenReferences();
-
-                screen->TestDanglingEnds();
-
-                m_frame->SetScreen( screen );
-                m_frame->UpdateHierarchyNavigator();
+                m_frame->DisplayCurrentSheet();
                 UpdateFind( ACTIONS::updateFind.MakeEvent() );
 
                 break;
@@ -766,6 +760,9 @@ int SCH_EDITOR_CONTROL::SimProbe( const TOOL_EVENT& aEvent )
     if( !simFrame )     // Defensive coding; shouldn't happen.
         return 0;
 
+    if( wxWindow* blocking_win = simFrame->Kiway().GetBlockingDialog() )
+        blocking_win->Close( true );
+
     // Deactivate other tools; particularly important if another PICKER is currently running
     Activate();
 
@@ -913,7 +910,12 @@ int SCH_EDITOR_CONTROL::SimTune( const TOOL_EVENT& aEvent )
                 KIWAY_PLAYER* simFrame = m_frame->Kiway().Player( FRAME_SIMULATOR, false );
 
                 if( simFrame )
+                {
+                    if( wxWindow* blocking_win = simFrame->Kiway().GetBlockingDialog() )
+                        blocking_win->Close( true );
+
                     static_cast<SIM_PLOT_FRAME*>( simFrame )->AddTuner( symbol );
+                }
 
                 // We do not really want to keep a symbol selected in schematic,
                 // so clear the current selection
@@ -1013,17 +1015,18 @@ static bool highlightNet( TOOL_MANAGER* aToolMgr, const VECTOR2D& aPosition )
         }
     }
 
-    if( !conn )
+    if( !conn || conn == editFrame->GetHighlightedConnection() )
     {
         editFrame->SetStatusText( wxEmptyString );
         editFrame->SendCrossProbeClearHighlight();
+        editFrame->SetHighlightedConnection( nullptr );
     }
     else
     {
         editFrame->SetCrossProbeConnection( conn );
+        editFrame->SetHighlightedConnection( conn );
     }
 
-    editFrame->SetHighlightedConnection( conn );
     editFrame->UpdateNetHighlightStatus();
 
     TOOL_EVENT dummy;
@@ -1834,7 +1837,8 @@ int SCH_EDITOR_CONTROL::Paste( const TOOL_EVENT& aEvent )
             if( !fn.IsAbsolute() )
             {
                 wxFileName currentSheetFileName = pasteRoot.LastScreen()->GetFileName();
-                fn.Normalize( wxPATH_NORM_ALL, currentSheetFileName.GetPath() );
+                fn.Normalize(  FN_NORMALIZE_FLAGS | wxPATH_NORM_ENV_VARS,
+                               currentSheetFileName.GetPath() );
             }
 
             // Try to find the screen for the pasted sheet by several means
@@ -1979,7 +1983,7 @@ int SCH_EDITOR_CONTROL::EditWithSymbolEditor( const TOOL_EVENT& aEvent )
     if( selection.GetSize() >= 1 )
         symbol = (SCH_SYMBOL*) selection.Front();
 
-    if( !symbol || symbol->GetEditFlags() != 0 )
+    if( !symbol || symbol->GetEditFlags() != 0 || symbol->IsMissingLibSymbol() )
         return 0;
 
     m_toolMgr->RunAction( ACTIONS::showSymbolEditor, true );
@@ -1987,6 +1991,9 @@ int SCH_EDITOR_CONTROL::EditWithSymbolEditor( const TOOL_EVENT& aEvent )
 
     if( symbolEditor )
     {
+        if( wxWindow* blocking_win = symbolEditor->Kiway().GetBlockingDialog() )
+            blocking_win->Close( true );
+
         if( aEvent.IsAction( &EE_ACTIONS::editWithLibEdit ) )
             symbolEditor->LoadSymbolFromSchematic( symbol );
         else if( aEvent.IsAction( &EE_ACTIONS::editLibSymbolWithLibEdit ) )
