@@ -438,6 +438,14 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
             chain_commands = true;
             break;
         }
+        else if( evt->IsAction( &EE_ACTIONS::rotateCW ) )
+        {
+            m_toolMgr->RunAction( EE_ACTIONS::rotateCW, true );
+        }
+        else if( evt->IsAction( &EE_ACTIONS::rotateCCW ) )
+        {
+            m_toolMgr->RunAction( EE_ACTIONS::rotateCCW, true );
+        }
         else if( evt->Action() == TA_CHOICE_MENU_CHOICE )
         {
             if( evt->GetCommandId().get() >= ID_POPUP_SCH_SELECT_UNIT_CMP
@@ -544,6 +552,7 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aOriginalItem, const wxPoin
     EE_RTREE&         items = m_frame->GetScreen()->Items();
     EE_RTREE::EE_TYPE itemsOverlapping = items.Overlapping( aOriginalItem->GetBoundingBox() );
     bool              ptHasUnselectedJunction = false;
+    SCH_LINE*         newWire = nullptr;
 
     for( SCH_ITEM* item : itemsOverlapping )
     {
@@ -625,19 +634,19 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aOriginalItem, const wxPoin
         case SCH_SHEET_T:
         case SCH_SYMBOL_T:
         case SCH_JUNCTION_T:
-            if( test->IsConnected( aPoint ) )
+            if( test->IsConnected( aPoint ) && !newWire )
             {
                 // Add a new wire between the symbol or junction and the selected item so
                 // the selected item can be dragged.
-                SCH_LINE* newWire = nullptr;
-
-                if( test->GetLayer() == LAYER_BUS_JUNCTION ||
-                    aOriginalItem->GetLayer() == LAYER_BUS )
+                if( test->GetLayer() == LAYER_BUS_JUNCTION
+                        || aOriginalItem->GetLayer() == LAYER_BUS )
                 {
                     newWire = new SCH_LINE( aPoint, LAYER_BUS );
                 }
                 else
+                {
                     newWire = new SCH_LINE( aPoint, LAYER_WIRE );
+                }
 
                 newWire->SetFlags( IS_NEW );
                 m_frame->AddToScreen( newWire, m_frame->GetScreen() );
@@ -685,6 +694,26 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aOriginalItem, const wxPoin
                     }
                 }
             }
+            else if( test->IsConnected( aPoint ) && !newWire )
+            {
+                // Add a new wire between the label and the selected item so the selected item
+                // can be dragged.
+                if( test->GetLayer() == LAYER_BUS_JUNCTION
+                        || aOriginalItem->GetLayer() == LAYER_BUS )
+                {
+                    newWire = new SCH_LINE( aPoint, LAYER_BUS );
+                }
+                else
+                {
+                    newWire = new SCH_LINE( aPoint, LAYER_WIRE );
+                }
+
+                newWire->SetFlags( IS_NEW );
+                m_frame->AddToScreen( newWire, m_frame->GetScreen() );
+
+                newWire->SetFlags( TEMP_SELECTED | STARTPOINT );
+                aList.push_back( newWire );
+            }
 
             break;
 
@@ -753,8 +782,8 @@ void SCH_MOVE_TOOL::moveItem( EDA_ITEM* aItem, const VECTOR2I& aDelta )
         if( aItem->HasFlag( ENDPOINT ) )
             line->MoveEnd( (wxPoint) aDelta );
 
-    }
         break;
+    }
 
     case SCH_PIN_T:
     case SCH_FIELD_T:
@@ -778,13 +807,16 @@ void SCH_MOVE_TOOL::moveItem( EDA_ITEM* aItem, const VECTOR2I& aDelta )
 
         break;
     }
+
     case SCH_SHEET_PIN_T:
     {
         SCH_SHEET_PIN* pin = (SCH_SHEET_PIN*) aItem;
+
         pin->SetStoredPos( pin->GetStoredPos() + (wxPoint) aDelta );
         pin->ConstrainOnEdge( pin->GetStoredPos() );
         break;
     }
+
     case SCH_LABEL_T:
     {
         SCH_TEXT* label = static_cast<SCH_TEXT*>( aItem );
@@ -802,6 +834,7 @@ void SCH_MOVE_TOOL::moveItem( EDA_ITEM* aItem, const VECTOR2I& aDelta )
 
         break;
     }
+
     default:
         static_cast<SCH_ITEM*>( aItem )->Move( (wxPoint) aDelta );
         break;
@@ -818,6 +851,17 @@ int SCH_MOVE_TOOL::AlignElements( const TOOL_EVENT& aEvent )
     EE_SELECTION& selection = m_selectionTool->RequestSelection( EE_COLLECTOR::MovableItems );
     bool append_undo = false;
 
+    auto doMoveItem =
+            [&]( EDA_ITEM* item, const VECTOR2I& delta )
+            {
+                saveCopyInUndoList( item, UNDO_REDO::CHANGED, append_undo );
+                append_undo = true;
+
+                moveItem( item, delta );
+                item->ClearFlags( IS_MOVING );
+                updateItem( item, true );
+            };
+
     for( SCH_ITEM* it : m_frame->GetScreen()->Items() )
     {
         if( !it->IsSelected() )
@@ -861,12 +905,7 @@ int SCH_MOVE_TOOL::AlignElements( const TOOL_EVENT& aEvent )
                         if( dragItem->GetParent() && dragItem->GetParent()->IsSelected() )
                             continue;
 
-                        saveCopyInUndoList( dragItem, UNDO_REDO::CHANGED, append_undo );
-                        append_undo = true;
-
-                        moveItem( dragItem, gridpt );
-                        dragItem->ClearFlags( IS_MOVING );
-                        updateItem( dragItem, true );
+                        doMoveItem( dragItem, gridpt );
                     }
                 }
             }
@@ -876,13 +915,49 @@ int SCH_MOVE_TOOL::AlignElements( const TOOL_EVENT& aEvent )
             VECTOR2I gridpt = grid.AlignGrid( item->GetPosition() ) - item->GetPosition();
 
             if( gridpt != VECTOR2I( 0, 0 ) )
-            {
-                saveCopyInUndoList( item, UNDO_REDO::CHANGED, append_undo );
-                append_undo = true;
+                doMoveItem( item, gridpt );
+        }
+        else if( item->Type() == SCH_SHEET_T )
+        {
+            SCH_SHEET* sheet = static_cast<SCH_SHEET*>( item );
+            VECTOR2I   topRight = sheet->GetPosition();
+            VECTOR2I   bottomRight = topRight + sheet->GetSize();
+            VECTOR2I   tl_gridpt = grid.AlignGrid( topRight ) - topRight;
+            VECTOR2I   br_gridpt = grid.AlignGrid( bottomRight ) - bottomRight;
 
-                moveItem( item, gridpt );
-                updateItem( item, true );
-                item->ClearFlags( IS_MOVING );
+            if( tl_gridpt != VECTOR2I( 0, 0 ) || br_gridpt != VECTOR2I( 0, 0 ) )
+            {
+                doMoveItem( sheet, tl_gridpt );
+
+                VECTOR2I newSize = (VECTOR2I) sheet->GetSize() - tl_gridpt + br_gridpt;
+                sheet->SetSize( wxSize( newSize.x, newSize.y ) );
+                updateItem( sheet, true );
+
+                for( SCH_SHEET_PIN* pin : sheet->GetPins() )
+                {
+                    VECTOR2I gridpt;
+
+                    if( pin->GetSide() == SHEET_SIDE::TOP || pin->GetSide() == SHEET_SIDE::LEFT )
+                        gridpt = tl_gridpt;
+                    else
+                        gridpt = br_gridpt;
+
+                    if( gridpt != VECTOR2I( 0, 0 ) )
+                    {
+                        EDA_ITEMS drag_items;
+                        getConnectedDragItems( pin, pin->GetConnectionPoints()[0], drag_items );
+
+                        doMoveItem( pin, gridpt );
+
+                        for( EDA_ITEM* dragItem : drag_items )
+                        {
+                            if( dragItem->GetParent() && dragItem->GetParent()->IsSelected() )
+                                continue;
+
+                            doMoveItem( dragItem, gridpt );
+                        }
+                    }
+                }
             }
         }
         else
@@ -918,12 +993,7 @@ int SCH_MOVE_TOOL::AlignElements( const TOOL_EVENT& aEvent )
                     if( dragItem->GetParent() && dragItem->GetParent()->IsSelected() )
                         continue;
 
-                    saveCopyInUndoList( dragItem, UNDO_REDO::CHANGED, append_undo );
-                    append_undo = true;
-
-                    moveItem( dragItem, most_common );
-                    dragItem->ClearFlags( IS_MOVING );
-                    updateItem( dragItem, true );
+                    doMoveItem( dragItem, most_common );
                 }
             }
         }
