@@ -498,25 +498,30 @@ void CN_CONNECTIVITY_ALGO::propagateConnections( BOARD_COMMIT* aCommit, PROPAGAT
 {
     bool skipConflicts = ( aMode == PROPAGATE_MODE::SKIP_CONFLICTS );
 
-    wxLogTrace( "CN", "propagateConnections: propagate skip conflicts? %d", skipConflicts );
+    wxLogTrace( wxT( "CN" ), wxT( "propagateConnections: propagate skip conflicts? %d" ),
+                skipConflicts );
 
     for( const auto& cluster : m_connClusters )
     {
         if( skipConflicts && cluster->IsConflicting() )
         {
-            wxLogTrace( "CN", "Conflicting nets in cluster %p; skipping update", cluster.get() );
+            wxLogTrace( wxT( "CN" ), wxT( "Conflicting nets in cluster %p; skipping update" ),
+                        cluster.get() );
         }
         else if( cluster->IsOrphaned() )
         {
-            wxLogTrace( "CN", "Skipping orphaned cluster %p [net: %s]", cluster.get(),
-                    (const char*) cluster->OriginNetName().c_str() );
+            wxLogTrace( wxT( "CN" ), wxT( "Skipping orphaned cluster %p [net: %s]" ),
+                        cluster.get(),
+                        (const char*) cluster->OriginNetName().c_str() );
         }
         else if( cluster->HasValidNet() )
         {
             if( cluster->IsConflicting() )
             {
-                wxLogTrace( "CN", "Conflicting nets in cluster %p; chose %d (%s)", cluster.get(),
-                            cluster->OriginNet(), cluster->OriginNetName() );
+                wxLogTrace( wxT( "CN" ), wxT( "Conflicting nets in cluster %p; chose %d (%s)" ),
+                            cluster.get(),
+                            cluster->OriginNet(),
+                            cluster->OriginNetName() );
             }
 
             // normal cluster: just propagate from the pads
@@ -542,15 +547,19 @@ void CN_CONNECTIVITY_ALGO::propagateConnections( BOARD_COMMIT* aCommit, PROPAGAT
 
             if( n_changed )
             {
-                wxLogTrace( "CN", "Cluster %p : net : %d %s", cluster.get(),
-                        cluster->OriginNet(), (const char*) cluster->OriginNetName().c_str() );
+                wxLogTrace( wxT( "CN" ), wxT( "Cluster %p : net : %d %s" ),
+                            cluster.get(),
+                            cluster->OriginNet(),
+                            (const char*) cluster->OriginNetName().c_str() );
             }
             else
-                wxLogTrace( "CN", "Cluster %p : nothing to propagate", cluster.get() );
+                wxLogTrace( wxT( "CN" ), wxT( "Cluster %p : nothing to propagate" ),
+                            cluster.get() );
         }
         else
         {
-            wxLogTrace( "CN", "Cluster %p : connected to unused net", cluster.get() );
+            wxLogTrace( wxT( "CN" ), wxT( "Cluster %p : connected to unused net" ),
+                        cluster.get() );
         }
     }
 }
@@ -590,7 +599,7 @@ void CN_CONNECTIVITY_ALGO::FindIsolatedCopperIslands( ZONE* aZone, PCB_LAYER_ID
         }
     }
 
-    wxLogTrace( "CN", "Found %u isolated islands\n", (unsigned)aIslands.size() );
+    wxLogTrace( wxT( "CN" ), wxT( "Found %u isolated islands\n" ), (unsigned) aIslands.size() );
 }
 
 void CN_CONNECTIVITY_ALGO::FindIsolatedCopperIslands( std::vector<CN_ZONE_ISOLATED_ISLAND_LIST>& aZones )
@@ -756,7 +765,9 @@ bool CN_VISITOR::operator()( CN_ITEM* aCandidate )
     if( parentA == parentB )
         return true;
 
-    if( !( parentA->GetLayerSet() & parentB->GetLayerSet() ).any() )
+    LSET commonLayers = parentA->GetLayerSet() & parentB->GetLayerSet();
+
+    if( !commonLayers.any() )
         return true;
 
     // If both m_item and aCandidate are marked dirty, they will both be searched
@@ -785,34 +796,35 @@ bool CN_VISITOR::operator()( CN_ITEM* aCandidate )
         return true;
     }
 
-    int accuracyA = 0;
-    int accuracyB = 0;
-
-    if( parentA->Type() == PCB_VIA_T
-            || parentA->Type() == PCB_TRACE_T
-            || parentA->Type() == PCB_ARC_T)
-        accuracyA = ( static_cast<const PCB_TRACK*>( parentA )->GetWidth() + 1 ) / 2;
-
-    if( parentB->Type() == PCB_VIA_T
-            || parentB->Type() == PCB_TRACE_T
-            || parentB->Type() == PCB_ARC_T )
-        accuracyB = ( static_cast<const PCB_TRACK*>( parentB )->GetWidth() + 1 ) / 2;
-
-    // Items do not necessarily have reciprocity as we only check for anchors
-    //  therefore, we check HitTest both directions A->B & B->A
-    for( int i = 0; i < aCandidate->AnchorCount(); ++i )
+    for( PCB_LAYER_ID layer : commonLayers.Seq() )
     {
-        if( parentB->HitTest( wxPoint( aCandidate->GetAnchor( i ) ), accuracyA ) )
+        FLASHING flashingA = FLASHING::NEVER_FLASHED;
+        FLASHING flashingB = FLASHING::NEVER_FLASHED;
+
+        if( const PAD* pad = dyn_cast<const PAD*>( parentA ) )
         {
-            m_item->Connect( aCandidate );
-            aCandidate->Connect( m_item );
-            return true;
+            if( !pad->GetRemoveUnconnected() || ( ( layer == F_Cu || layer == B_Cu ) && pad->GetKeepTopBottom() ) )
+                flashingA = FLASHING::ALWAYS_FLASHED;
+        }
+        else if( const PCB_VIA* via = dyn_cast<const PCB_VIA*>( parentA ) )
+        {
+            if( !via->GetRemoveUnconnected() || ( ( layer == F_Cu || layer == B_Cu ) && via->GetKeepTopBottom() ) )
+                flashingA = FLASHING::ALWAYS_FLASHED;
         }
-    }
 
-    for( int i = 0; i < m_item->AnchorCount(); ++i )
-    {
-        if( parentA->HitTest( wxPoint( m_item->GetAnchor( i ) ), accuracyB ) )
+        if( const PAD* pad = dyn_cast<const PAD*>( parentB ) )
+        {
+            if( !pad->GetRemoveUnconnected() || ( ( layer == F_Cu || layer == B_Cu ) && pad->GetKeepTopBottom() ) )
+                flashingB = FLASHING::ALWAYS_FLASHED;
+        }
+        else if( const PCB_VIA* via = dyn_cast<const PCB_VIA*>( parentB ) )
+        {
+            if( !via->GetRemoveUnconnected() || ( ( layer == F_Cu || layer == B_Cu ) && via->GetKeepTopBottom() ) )
+                flashingB = FLASHING::ALWAYS_FLASHED;
+        }
+
+        if( parentA->GetEffectiveShape( layer, flashingA )->Collide(
+                parentB->GetEffectiveShape( layer, flashingB ).get() ) )
         {
             m_item->Connect( aCandidate );
             aCandidate->Connect( m_item );
