@@ -29,10 +29,11 @@
 #include <math/util.h>      // for KiROUND
 #include <eda_draw_frame.h>
 #include <geometry/shape_circle.h>
+#include <geometry/shape_compound.h>
+#include <geometry/shape_null.h>
+#include <geometry/shape_rect.h>
 #include <geometry/shape_segment.h>
 #include <geometry/shape_simple.h>
-#include <geometry/shape_rect.h>
-#include <geometry/shape_compound.h>
 #include <string_utils.h>
 #include <i18n_utility.h>
 #include <view/view.h>
@@ -200,7 +201,7 @@ bool PAD::IsFlipped() const
 
 bool PAD::FlashLayer( LSET aLayers ) const
 {
-    for( auto layer : aLayers.Seq() )
+    for( PCB_LAYER_ID layer : aLayers.Seq() )
     {
         if( FlashLayer( layer ) )
             return true;
@@ -215,31 +216,14 @@ bool PAD::FlashLayer( int aLayer ) const
     std::vector<KICAD_T> types
     { PCB_TRACE_T, PCB_ARC_T, PCB_VIA_T, PCB_PAD_T, PCB_ZONE_T, PCB_FP_ZONE_T };
 
-    const BOARD* board = GetBoard();
+    if( aLayer != UNDEFINED_LAYER && !IsOnLayer( static_cast<PCB_LAYER_ID>( aLayer ) ) )
+        return false;
 
-    switch( GetAttribute() )
+    if( aLayer == UNDEFINED_LAYER )
+        return true;
+
+    if( GetAttribute() == PAD_ATTRIB::NPTH && IsCopperLayer( aLayer ) )
     {
-    case PAD_ATTRIB::PTH:
-        if( aLayer == UNDEFINED_LAYER )
-            return true;
-
-        /// Heat sink pads always get copper
-        if( GetProperty() == PAD_PROP::HEATSINK )
-            return IsOnLayer( static_cast<PCB_LAYER_ID>( aLayer ) );
-
-        if( !m_removeUnconnectedLayer )
-            return IsOnLayer( static_cast<PCB_LAYER_ID>( aLayer ) );
-
-        // Plated through hole pads need copper on the top/bottom layers for proper soldering
-        // Unless the user has removed them in the pad dialog
-        if( m_keepTopBottomLayer && ( aLayer == F_Cu || aLayer == B_Cu ) )
-            return IsOnLayer( static_cast<PCB_LAYER_ID>( aLayer ) );
-
-        return board && board->GetConnectivity()->IsConnectedOnLayer( this,
-                                                                      static_cast<int>( aLayer ),
-                                                                      types );
-
-    case PAD_ATTRIB::NPTH:
         if( GetShape() == PAD_SHAPE::CIRCLE && GetDrillShape() == PAD_DRILL_SHAPE_CIRCLE )
         {
             if( GetOffset() == wxPoint( 0, 0 ) && GetDrillSize().x >= GetSize().x )
@@ -253,17 +237,32 @@ bool PAD::FlashLayer( int aLayer ) const
                 return false;
             }
         }
+    }
 
-        KI_FALLTHROUGH;
+    if( LSET::FrontBoardTechMask().test( aLayer ) )
+        aLayer = F_Cu;
+    else if( LSET::BackBoardTechMask().test( aLayer ) )
+        aLayer = B_Cu;
 
-    case PAD_ATTRIB::SMD:
-    case PAD_ATTRIB::CONN:
-    default:
-        if( aLayer == UNDEFINED_LAYER )
+    if( GetAttribute() == PAD_ATTRIB::PTH && IsCopperLayer( aLayer ) )
+    {
+        /// Heat sink pads always get copper
+        if( GetProperty() == PAD_PROP::HEATSINK )
             return true;
 
-        return IsOnLayer( static_cast<PCB_LAYER_ID>( aLayer ) );
+        if( !m_removeUnconnectedLayer )
+            return true;
+
+        // Plated through hole pads need copper on the top/bottom layers for proper soldering
+        // Unless the user has removed them in the pad dialog
+        if( m_keepTopBottomLayer && ( aLayer == F_Cu || aLayer == B_Cu ) )
+            return true;
+
+        if( const BOARD* board = GetBoard() )
+            return board->GetConnectivity()->IsConnectedOnLayer( this, aLayer, types, true );
     }
+
+    return true;
 }
 
 
@@ -307,8 +306,37 @@ const std::shared_ptr<SHAPE_POLY_SET>& PAD::GetEffectivePolygon() const
 }
 
 
-std::shared_ptr<SHAPE> PAD::GetEffectiveShape( PCB_LAYER_ID aLayer ) const
+std::shared_ptr<SHAPE> PAD::GetEffectiveShape( PCB_LAYER_ID aLayer, FLASHING flashPTHPads ) const
 {
+    if( aLayer == Edge_Cuts )
+    {
+        if( GetAttribute() == PAD_ATTRIB::PTH || GetAttribute() == PAD_ATTRIB::NPTH )
+            return std::make_shared<SHAPE_SEGMENT>( *GetEffectiveHoleShape() );
+        else
+            return std::make_shared<SHAPE_NULL>();
+    }
+
+    if( GetAttribute() == PAD_ATTRIB::PTH )
+    {
+        bool flash;
+
+        if( flashPTHPads == FLASHING::NEVER_FLASHED )
+            flash = false;
+        else if( flashPTHPads == FLASHING::ALWAYS_FLASHED )
+            flash = true;
+        else
+            flash = FlashLayer( aLayer );
+
+        if( !flash )
+        {
+            if( GetAttribute() == PAD_ATTRIB::PTH )
+                return std::make_shared<SHAPE_SEGMENT>( *GetEffectiveHoleShape() );
+            else
+                return std::make_shared<SHAPE_NULL>();
+        }
+    }
+
+
     if( m_shapesDirty )
         BuildEffectiveShapes( aLayer );
 
@@ -470,7 +498,7 @@ void PAD::BuildEffectiveShapes( PCB_LAYER_ID aLayer ) const
         break;
 
     default:
-        wxFAIL_MSG( "PAD::buildEffectiveShapes: Unsupported pad shape: "
+        wxFAIL_MSG( wxT( "PAD::buildEffectiveShapes: Unsupported pad shape: " )
                     + PAD_SHAPE_T_asString( effectiveShape ) );
         break;
     }
@@ -724,6 +752,32 @@ int PAD::GetLocalClearance( wxString* aSource ) const
 }
 
 
+int PAD::GetOwnClearance( PCB_LAYER_ID aLayer, wxString* aSource ) const
+{
+    DRC_CONSTRAINT c;
+
+    if( GetBoard() && GetBoard()->GetDesignSettings().m_DRCEngine )
+    {
+        BOARD_DESIGN_SETTINGS& bds = GetBoard()->GetDesignSettings();
+
+        if( GetAttribute() == PAD_ATTRIB::NPTH )
+            c = bds.m_DRCEngine->EvalRules( HOLE_CLEARANCE_CONSTRAINT, this, nullptr, aLayer );
+        else
+            c = bds.m_DRCEngine->EvalRules( CLEARANCE_CONSTRAINT, this, nullptr, aLayer );
+    }
+
+    if( c.Value().HasMin() )
+    {
+        if( aSource )
+            *aSource = c.GetName();
+
+        return c.Value().Min();
+    }
+
+    return 0;
+}
+
+
 int PAD::GetSolderMaskMargin() const
 {
     // The pad inherits the margin only to calculate a default shape,
@@ -963,13 +1017,13 @@ void PAD::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>&
         if( GetDrillShape() == PAD_DRILL_SHAPE_CIRCLE )
         {
             aList.emplace_back( _( "Hole" ),
-                                wxString::Format( "%s",
+                                wxString::Format( wxT( "%s" ),
                                                   MessageTextFromValue( units, m_drill.x ) ) );
         }
         else
         {
             aList.emplace_back( _( "Hole X / Y" ),
-                                wxString::Format( "%s / %s",
+                                wxString::Format( wxT( "%s / %s" ),
                                                   MessageTextFromValue( units, m_drill.x ),
                                                   MessageTextFromValue( units, m_drill.y ) ) );
         }
@@ -987,7 +1041,7 @@ void PAD::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>&
     }
 #if 0
     // useful for debug only
-    aList.emplace_back( "UUID", m_Uuid.AsString() );
+    aList.emplace_back( wxT( "UUID" ), m_Uuid.AsString() );
 #endif
 }
 
@@ -1295,8 +1349,8 @@ void PAD::ViewGetLayers( int aLayers[], int& aCount ) const
     {
         wxString msg;
         msg.Printf( wxT( "footprint %s, pad %s: could not find valid layer for pad" ),
-                GetParent() ? GetParent()->GetReference() : "<null>",
-                GetNumber().IsEmpty() ? "(unnumbered)" : GetNumber() );
+                    GetParent() ? GetParent()->GetReference() : wxT( "<null>" ),
+                    GetNumber().IsEmpty() ? wxT( "(unnumbered)" ) : GetNumber() );
         wxLogDebug( msg );
     }
 #endif
@@ -1318,7 +1372,14 @@ double PAD::ViewGetLOD( int aLayer, KIGFX::VIEW* aView ) const
 
     // Handle board visibility
     if( board )
-        visible = board->GetVisibleLayers() & board->GetEnabledLayers();
+        visible &= board->GetEnabledLayers();
+
+    // Handle view visibility
+    for( int layer = 0; layer < PCB_LAYER_ID_COUNT; ++layer )
+    {
+        if( !aView->IsLayerVisible( layer ) )
+            visible.set( layer, false );
+    }
 
     // Handle Render tab switches
     if( ( GetAttribute() == PAD_ATTRIB::PTH || GetAttribute() == PAD_ATTRIB::NPTH )
@@ -1532,7 +1593,7 @@ void PAD::TransformShapeWithClearanceToPolygon( SHAPE_POLY_SET& aCornerBuffer,
                                                 int aError, ERROR_LOC aErrorLoc,
                                                 bool ignoreLineWidth ) const
 {
-    wxASSERT_MSG( !ignoreLineWidth, "IgnoreLineWidth has no meaning for pads." );
+    wxASSERT_MSG( !ignoreLineWidth, wxT( "IgnoreLineWidth has no meaning for pads." ) );
 
     // minimal segment count to approximate a circle to create the polygonal pad shape
     // This minimal value is mainly for very small pads, like SM0402.
@@ -1626,7 +1687,7 @@ void PAD::TransformShapeWithClearanceToPolygon( SHAPE_POLY_SET& aCornerBuffer,
     }
 
     default:
-        wxFAIL_MSG( "PAD::TransformShapeWithClearanceToPolygon no implementation for "
+        wxFAIL_MSG( wxT( "PAD::TransformShapeWithClearanceToPolygon no implementation for " )
                     + PAD_SHAPE_T_asString( GetShape() ) );
         break;
     }
