@@ -390,6 +390,8 @@ public:
     int GetLocalClearance() const               { return m_localClearance; }
     void SetLocalClearance( int aClearance )    { m_localClearance = aClearance; }
 
+    int GetOwnClearance( PCB_LAYER_ID aLayer, wxString* aSource = nullptr ) const override;
+
     int GetLocalSolderPasteMargin() const       { return m_localSolderPasteMargin; }
     void SetLocalSolderPasteMargin( int aMargin ) { m_localSolderPasteMargin = aMargin; }
 
@@ -423,8 +425,9 @@ public:
                                               int aError, ERROR_LOC aErrorLoc ) const;
 
     // @copydoc BOARD_ITEM::GetEffectiveShape
-    virtual std::shared_ptr<SHAPE> GetEffectiveShape( PCB_LAYER_ID aLayer = UNDEFINED_LAYER,
-            FLASHING aFlash = FLASHING::DEFAULT ) const override;
+    virtual std::shared_ptr<SHAPE>
+    GetEffectiveShape( PCB_LAYER_ID aLayer = UNDEFINED_LAYER,
+                       FLASHING flashPTHPads = FLASHING::DEFAULT ) const override;
 
     const std::shared_ptr<SHAPE_POLY_SET>& GetEffectivePolygon() const;
 
