@@ -195,7 +195,7 @@ bool calcIsInsideCourtyard( BOARD_ITEM* aItem, const EDA_RECT& aItemBBox,
 {
     SHAPE_POLY_SET footprintCourtyard;
 
-    footprintCourtyard = aFootprint->GetPolyCourtyard( aSide );
+    footprintCourtyard = aFootprint->GetCourtyard( aSide );
 
     if( !aFootprint->GetBoundingBox().Intersects( aItemBBox ) )
         return false;
@@ -495,7 +495,7 @@ bool calcIsInsideArea( BOARD_ITEM* aItem, const EDA_RECT& aItemBBox, PCB_EXPR_CO
 
         if( ( aArea->GetLayerSet() & LSET::FrontMask() ).any() )
         {
-            SHAPE_POLY_SET courtyard = footprint->GetPolyCourtyard( F_CrtYd );
+            const SHAPE_POLY_SET& courtyard = footprint->GetCourtyard( F_CrtYd );
 
             if( courtyard.OutlineCount() == 0 )
             {
@@ -512,7 +512,7 @@ bool calcIsInsideArea( BOARD_ITEM* aItem, const EDA_RECT& aItemBBox, PCB_EXPR_CO
 
         if( ( aArea->GetLayerSet() & LSET::BackMask() ).any() )
         {
-            SHAPE_POLY_SET courtyard = footprint->GetPolyCourtyard( B_CrtYd );
+            const SHAPE_POLY_SET& courtyard = footprint->GetCourtyard( B_CrtYd );
 
             if( courtyard.OutlineCount() == 0 )
             {
@@ -576,10 +576,11 @@ bool isInsideArea( BOARD_ITEM* aItem, const EDA_RECT& aItemBBox, PCB_EXPR_CONTEX
     if( !aArea || aArea == aItem || aArea->GetParent() == aItem )
         return false;
 
-    BOARD*                              board = aArea->GetBoard();
-    std::unique_lock<std::mutex>        cacheLock( board->m_CachesMutex );
-    std::pair<BOARD_ITEM*, BOARD_ITEM*> key( aArea, aItem );
-    auto                                i = board->m_InsideAreaCache.find( key );
+    BOARD*                                             board = aArea->GetBoard();
+    std::unique_lock<std::mutex>                       cacheLock( board->m_CachesMutex );
+    PCB_LAYER_ID                                       layer = aCtx->GetLayer();
+    std::tuple<BOARD_ITEM*, BOARD_ITEM*, PCB_LAYER_ID> key( aArea, aItem, layer );
+    auto                                               i = board->m_InsideAreaCache.find( key );
 
     if( i != board->m_InsideAreaCache.end() )
         return i->second;
@@ -782,20 +783,27 @@ static void isCoupledDiffPair( LIBEVAL::CONTEXT* aCtx, void* self )
     aCtx->Push( result );
 
     result->SetDeferredEval(
-            [a, b]() -> double
+            [a, b, context]() -> double
             {
                 NETINFO_ITEM* netinfo = a ? a->GetNet() : nullptr;
-                wxString      coupledNet;
-                wxString      dummy;
 
-                if( netinfo
-                        && DRC_ENGINE::MatchDpSuffix( netinfo->GetNetname(), coupledNet, dummy )
-                        && ( !b || b->GetNetname() == coupledNet ) )
+                if( !netinfo )
+                    return 0.0;
+
+                wxString coupledNet;
+                wxString dummy;
+
+                if( !DRC_ENGINE::MatchDpSuffix( netinfo->GetNetname(), coupledNet, dummy ) )
+                    return 0.0;
+
+                if( context->GetConstraint() == DRC_CONSTRAINT_T::LENGTH_CONSTRAINT
+                        || context->GetConstraint() == DRC_CONSTRAINT_T::SKEW_CONSTRAINT )
                 {
+                    // DRC engine evaluates these singly, so we won't have a B item
                     return 1.0;
                 }
 
-                return 0.0;
+                return b && b->GetNetname() == coupledNet;
             } );
 }
 
@@ -1184,12 +1192,12 @@ PCB_EXPR_EVALUATOR::~PCB_EXPR_EVALUATOR()
 bool PCB_EXPR_EVALUATOR::Evaluate( const wxString& aExpr )
 {
     PCB_EXPR_UCODE   ucode;
-    PCB_EXPR_CONTEXT preflightContext( F_Cu );
+    PCB_EXPR_CONTEXT preflightContext( NULL_CONSTRAINT, F_Cu );
 
     if( !m_compiler.Compile( aExpr.ToUTF8().data(), &ucode, &preflightContext ) )
         return false;
 
-    PCB_EXPR_CONTEXT evaluationContext( F_Cu );
+    PCB_EXPR_CONTEXT evaluationContext( NULL_CONSTRAINT, F_Cu );
     LIBEVAL::VALUE*  result = ucode.Run( &evaluationContext );
 
     if( result->GetType() == LIBEVAL::VT_NUMERIC )
