@@ -312,10 +312,9 @@ COLOR4D PCB_RENDER_SETTINGS::GetColor( const VIEW_ITEM* aItem, int aLayer ) cons
         }
 
         if( netColor == COLOR4D::UNSPECIFIED )
-        {
             netColor = color;
-        }
-        else if( selected )
+
+        if( selected )
         {
             // Selection brightening overrides highlighting
             netColor.Brighten( m_selectFactor );
@@ -828,29 +827,37 @@ void PCB_PAINTER::draw( const PCB_VIA* aVia, int aLayer )
     default:                    wxASSERT( false );                                           break;
     }
 
+    m_gal->SetStrokeColor( color );
+    m_gal->SetFillColor( color );
+    m_gal->SetIsStroke( true );
+    m_gal->SetIsFill( false );
+
     if( sketchMode )
-    {
-        // Outline mode
-        m_gal->SetIsStroke( true );
-        m_gal->SetIsFill( false );
         m_gal->SetLineWidth( m_pcbSettings.m_outlineWidth );
-        m_gal->SetStrokeColor( color );
-    }
-    else
-    {
-        // Filled mode
-        m_gal->SetIsFill( true );
-        m_gal->SetIsStroke( false );
-        m_gal->SetFillColor( color );
-    }
 
     if( aLayer == LAYER_VIA_HOLES )
     {
+        m_gal->SetIsStroke( false );
+        m_gal->SetIsFill( true );
         m_gal->DrawCircle( center, getDrillSize( aVia ) / 2.0 );
     }
-    else if( aLayer == LAYER_VIA_THROUGH || m_pcbSettings.GetDrawIndividualViaLayers() )
+    else if( aLayer == LAYER_VIA_THROUGH || m_pcbSettings.IsPrinting() )
     {
-        m_gal->DrawCircle( center, aVia->GetWidth() / 2.0 );
+        int    annular_width = ( aVia->GetWidth() - getDrillSize( aVia ) ) / 2.0;
+        double radius = aVia->GetWidth() / 2.0;
+        bool   draw = aLayer == LAYER_VIA_THROUGH;
+
+        if( m_pcbSettings.IsPrinting() )
+            draw = aVia->FlashLayer( m_pcbSettings.GetPrintLayers() );
+
+        if( !sketchMode )
+        {
+            m_gal->SetLineWidth( annular_width );
+            radius -= annular_width / 2.0;
+        }
+
+        if( draw )
+            m_gal->DrawCircle( center, radius );
     }
     else if( aLayer == LAYER_VIA_BBLIND || aLayer == LAYER_VIA_MICROVIA )
     {
@@ -862,7 +869,11 @@ void PCB_PAINTER::draw( const PCB_VIA* aVia, int aLayer )
         double radius =  aVia->GetWidth() / 2.0;
 
         if( !sketchMode )
+        {
+            m_gal->SetIsFill( true );
+            m_gal->SetIsStroke( false );
             m_gal->SetLineWidth( ( aVia->GetWidth() - aVia->GetDrillValue() ) / 2.0 );
+        }
 
         m_gal->DrawArc( center, radius, M_PI * -0.375, M_PI * 0.375 );
         m_gal->DrawArc( center, radius, M_PI * 0.625, M_PI * 1.375 );
@@ -907,123 +918,125 @@ void PCB_PAINTER::draw( const PCB_VIA* aVia, int aLayer )
 
 void PCB_PAINTER::draw( const PAD* aPad, int aLayer )
 {
-    COLOR4D                color = m_pcbSettings.GetColor( aPad, aLayer );
+    COLOR4D color = m_pcbSettings.GetColor( aPad, aLayer );
 
     if( IsNetnameLayer( aLayer ) )
     {
-        // Is anything that we can display enabled?
-        if( m_pcbSettings.m_netNamesOnPads || m_pcbSettings.m_padNumbers )
+        wxString                          netname;
+        wxString                          padNumber;
+
+        if( m_pcbSettings.m_padNumbers )
+            padNumber = UnescapeString( aPad->GetNumber() );
+
+        if( m_pcbSettings.m_netNamesOnPads )
+            netname = UnescapeString( aPad->GetShortNetname() );
+
+        if( aPad->GetBoard()->IsElementVisible( LAYER_NO_CONNECTS )
+                && aPad->GetShortNetname().StartsWith( wxT( "unconnected-(" ) ) )
         {
-            bool displayNetname = ( m_pcbSettings.m_netNamesOnPads && !aPad->GetNetname().empty() );
-            EDA_RECT padBBox = aPad->GetBoundingBox();
-            VECTOR2D position = padBBox.Centre();
-            VECTOR2D padsize = VECTOR2D( padBBox.GetSize() );
+            wxString pinType = aPad->GetPinType();
 
-            if( aPad->GetShape() != PAD_SHAPE::CUSTOM )
-            {
-                // Don't allow a 45º rotation to bloat a pad's bounding box unnecessarily
-                double limit = std::min( aPad->GetSize().x, aPad->GetSize().y ) * 1.1;
-
-                if( padsize.x > limit && padsize.y > limit )
-                {
-                    padsize.x = limit;
-                    padsize.y = limit;
-                }
-            }
-
-            double maxSize = PCB_RENDER_SETTINGS::MAX_FONT_SIZE;
-            double size = padsize.y;
-
-            m_gal->Save();
-            m_gal->Translate( position );
-
-            // Keep the size ratio for the font, but make it smaller
-            if( padsize.x < padsize.y )
-            {
-                m_gal->Rotate( DECIDEG2RAD( -900.0 ) );
-                size = padsize.x;
-                std::swap( padsize.x, padsize.y );
-            }
-
-            // Font size limits
-            if( size > maxSize )
-                size = maxSize;
-
-            // Default font settings
-            m_gal->SetHorizontalJustify( GR_TEXT_HJUSTIFY_CENTER );
-            m_gal->SetVerticalJustify( GR_TEXT_VJUSTIFY_CENTER );
-            m_gal->SetFontBold( false );
-            m_gal->SetFontItalic( false );
-            m_gal->SetFontUnderlined( false );
-            m_gal->SetTextMirrored( false );
-            m_gal->SetStrokeColor( m_pcbSettings.GetColor( aPad, aLayer ) );
-            m_gal->SetIsStroke( true );
-            m_gal->SetIsFill( false );
-
-            // We have already translated the GAL to be centered at the center of the pad's
-            // bounding box
-            VECTOR2D textpos( 0.0, 0.0 );
-
-            // Divide the space, to display both pad numbers and netnames and set the Y text
-            // position to display 2 lines
-            if( displayNetname && m_pcbSettings.m_padNumbers )
-            {
-                size = size / 2.5;
-                textpos.y = size / 1.7;
-            }
-
-            if( displayNetname )
-            {
-                wxString netname = UnescapeString( aPad->GetShortNetname() );
-                wxString pinType = aPad->GetPinType();
-
-                // If the pad is actually not connected (unique pad in the net),
-                // shorten the displayed netname (actual name not useful)
-                // Can happen if the pad netname is edited inside the board editor, therefore
-                // having a netname not coming from schematic
-                if( netname.StartsWith( "unconnected-(" ) )
-                {
-                    if( pinType == wxT( "no_connect" ) || pinType.EndsWith( wxT( "+no_connect" ) ) )
-                        netname = "x";
-                    else if( pinType == wxT( "free" ) )
-                        netname = "*";
-                }
-
-                // approximate the size of net name text:
-                double tsize = 1.5 * padsize.x / std::max( PrintableCharCount( netname ), 1 );
-                tsize = std::min( tsize, size );
-
-                // Use a smaller text size to handle interline, pen size...
-                tsize *= 0.7;
-                VECTOR2D namesize( tsize, tsize );
-
-                m_gal->SetGlyphSize( namesize );
-                m_gal->SetLineWidth( namesize.x / 12.0 );
-                m_gal->BitmapText( netname, textpos, 0.0 );
-            }
-
-            if( m_pcbSettings.m_padNumbers )
-            {
-                const wxString& padNumber = aPad->GetNumber();
-                textpos.y = -textpos.y;
-
-                // approximate the size of the pad number text:
-                double tsize = 1.5 * padsize.x / std::max( PrintableCharCount( padNumber ), 1 );
-                tsize = std::min( tsize, size );
-
-                // Use a smaller text size to handle interline, pen size...
-                tsize *= 0.7;
-                tsize = std::min( tsize, size );
-                VECTOR2D numsize( tsize, tsize );
-
-                m_gal->SetGlyphSize( numsize );
-                m_gal->SetLineWidth( numsize.x / 12.0 );
-                m_gal->BitmapText( padNumber, textpos, 0.0 );
-            }
-
-            m_gal->Restore();
+            if( pinType == wxT( "no_connect" ) || pinType.EndsWith( wxT( "+no_connect" ) ) )
+                netname = wxT( "x" );
+            else if( pinType == wxT( "free" ) )
+                netname = wxT( "*" );
         }
 
+        if( netname.IsEmpty() && padNumber.IsEmpty() )
+            return;
+
+        EDA_RECT padBBox = aPad->GetBoundingBox();
+        VECTOR2D position = padBBox.Centre();
+        VECTOR2D padsize = VECTOR2D( padBBox.GetSize() );
+
+        if( aPad->GetShape() != PAD_SHAPE::CUSTOM )
+        {
+            // Don't allow a 45° rotation to bloat a pad's bounding box unnecessarily
+            double limit = std::min( aPad->GetSize().x, aPad->GetSize().y ) * 1.1;
+
+            if( padsize.x > limit && padsize.y > limit )
+            {
+                padsize.x = limit;
+                padsize.y = limit;
+            }
+        }
+
+        double maxSize = PCB_RENDER_SETTINGS::MAX_FONT_SIZE;
+        double size = padsize.y;
+
+        m_gal->Save();
+        m_gal->Translate( position );
+
+        // Keep the size ratio for the font, but make it smaller
+        if( padsize.x < padsize.y )
+        {
+            m_gal->Rotate( DECIDEG2RAD( -900.0 ) );
+            size = padsize.x;
+            std::swap( padsize.x, padsize.y );
+        }
+
+        // Font size limits
+        if( size > maxSize )
+            size = maxSize;
+
+        // Default font settings
+        m_gal->SetHorizontalJustify( GR_TEXT_HJUSTIFY_CENTER );
+        m_gal->SetVerticalJustify( GR_TEXT_VJUSTIFY_CENTER );
+        m_gal->SetFontBold( false );
+        m_gal->SetFontItalic( false );
+        m_gal->SetFontUnderlined( false );
+        m_gal->SetTextMirrored( false );
+        m_gal->SetStrokeColor( m_pcbSettings.GetColor( aPad, aLayer ) );
+        m_gal->SetIsStroke( true );
+        m_gal->SetIsFill( false );
+
+        // We have already translated the GAL to be centered at the center of the pad's
+        // bounding box
+        VECTOR2D textpos( 0.0, 0.0 );
+
+        // Divide the space, to display both pad numbers and netnames and set the Y text
+        // position to display 2 lines
+        if( !netname.IsEmpty() && !padNumber.IsEmpty() )
+        {
+            size = size / 2.5;
+            textpos.y = size / 1.7;
+        }
+
+        if( !netname.IsEmpty() )
+        {
+            // approximate the size of net name text:
+            double tsize = 1.5 * padsize.x / std::max( PrintableCharCount( netname ), 1 );
+            tsize = std::min( tsize, size );
+
+            // Use a smaller text size to handle interline, pen size...
+            tsize *= 0.7;
+            VECTOR2D namesize( tsize, tsize );
+
+            m_gal->SetGlyphSize( namesize );
+            m_gal->SetLineWidth( namesize.x / 12.0 );
+            m_gal->BitmapText( netname, textpos, 0.0 );
+        }
+
+        if( !padNumber.IsEmpty() )
+        {
+            textpos.y = -textpos.y;
+
+            // approximate the size of the pad number text:
+            double tsize = 1.5 * padsize.x / std::max( PrintableCharCount( padNumber ), 1 );
+            tsize = std::min( tsize, size );
+
+            // Use a smaller text size to handle interline, pen size...
+            tsize *= 0.7;
+            tsize = std::min( tsize, size );
+            VECTOR2D numsize( tsize, tsize );
+
+            m_gal->SetGlyphSize( numsize );
+            m_gal->SetLineWidth( numsize.x / 12.0 );
+            m_gal->BitmapText( padNumber, textpos, 0.0 );
+        }
+
+        m_gal->Restore();
+
         return;
     }
     else if( aLayer == LAYER_PAD_HOLEWALLS )
@@ -1302,6 +1315,9 @@ void PCB_PAINTER::draw( const PAD* aPad, int aLayer )
 
         if( flashActiveLayer || aPad->GetDrillSize().x )
         {
+            if( aPad->GetAttribute() == PAD_ATTRIB::NPTH )
+                color = m_pcbSettings.GetLayerColor( LAYER_NON_PLATEDHOLES );
+
             m_gal->SetLineWidth( m_pcbSettings.m_outlineWidth );
             m_gal->SetIsStroke( true );
             m_gal->SetIsFill( false );
@@ -1743,10 +1759,10 @@ void PCB_PAINTER::draw( const ZONE* aZone, int aLayer )
 
         // Draw each contour (main contour and holes)
 
-        /* This line:
+        /*
          * m_gal->DrawPolygon( *outline );
-         * should be enough, but currently does not work to draw holes contours in a complex polygon
-         * so each contour is draw as a simple polygon
+         * should be enough, but currently does not work to draw holes contours in a complex
+         * polygon so each contour is draw as a simple polygon
          */
 
         // Draw the main contour
@@ -1911,6 +1927,8 @@ void PCB_PAINTER::draw( const PCB_MARKER* aMarker, int aLayer )
         return;
     }
 
+    const_cast<PCB_MARKER*>( aMarker )->SetZoom( 1.0 / sqrt( m_gal->GetZoomFactor() ) );
+
     SHAPE_LINE_CHAIN polygon;
     aMarker->ShapeToPolygon( polygon );
 
