@@ -2,7 +2,7 @@
  * KiRouter - a push-and-(sometimes-)shove PCB router
  *
  * Copyright (C) 2013-2019 CERN
- * Copyright (C) 2016-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2016-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
@@ -712,6 +712,24 @@ void NODE::Add( std::unique_ptr< ITEM > aItem, bool aAllowRedundant )
 }
 
 
+void NODE::AddEdgeExclusion( std::unique_ptr<SHAPE> aShape )
+{
+    m_edgeExclusions.push_back( std::move( aShape ) );
+}
+
+
+bool NODE::QueryEdgeExclusions( const VECTOR2I& aPos ) const
+{
+    for( const std::unique_ptr<SHAPE>& edgeExclusion : m_edgeExclusions )
+    {
+        if( edgeExclusion->Collide( aPos ) )
+            return true;
+    }
+
+    return false;
+}
+
+
 void NODE::doRemove( ITEM* aItem )
 {
     // case 1: removing an item that is stored in the root node from any branch:
@@ -1033,6 +1051,10 @@ const LINE NODE::AssembleLine( LINKED_ITEM* aSeg, int* aOriginSegmentIndex,
     // Remove duplicate verts, but do NOT remove colinear segments here!
     pl.Line().Simplify( false );
 
+    // TODO: maintain actual segment index under simplifcation system
+    if( aOriginSegmentIndex && *aOriginSegmentIndex >= pl.SegmentCount() )
+        *aOriginSegmentIndex = pl.SegmentCount() - 1;
+
     assert( pl.SegmentCount() != 0 );
 
     return pl;
