@@ -46,6 +46,7 @@
 #include <tools/tool_event_utils.h>
 #include <tools/pcb_grid_helper.h>
 #include <tools/pad_tool.h>
+#include <tools/drawing_tool.h>
 #include <view/view_controls.h>
 #include <connectivity/connectivity_algo.h>
 #include <connectivity/connectivity_items.h>
@@ -262,12 +263,11 @@ bool EDIT_TOOL::invokeInlineRouter( int aDragMode )
 }
 
 
-
 bool EDIT_TOOL::isRouterActive() const
 {
     ROUTER_TOOL* router = m_toolMgr->GetTool<ROUTER_TOOL>();
 
-    return router && router->IsToolActive();
+    return router && router->RoutingInProgress();
 }
 
 
@@ -289,6 +289,22 @@ int EDIT_TOOL::Drag( const TOOL_EVENT& aEvent )
             {
                 sTool->FilterCollectorForFreePads( aCollector );
                 sTool->FilterCollectorForHierarchy( aCollector, true );
+
+                // drop a knee between two segments to a single segment
+                if( aCollector.GetCount() == 2 && dynamic_cast<PCB_TRACK*>( aCollector[0] ) )
+                {
+                    static KICAD_T types[] = { PCB_VIA_T, PCB_TRACE_T, PCB_ARC_T, EOT };
+
+                    PCB_TRACK*  a = static_cast<PCB_TRACK*>( aCollector[0] );
+                    PCB_TRACK*  b = static_cast<PCB_TRACK*>( aCollector[1] );
+                    const auto& c = aCollector[0]->GetBoard()->GetConnectivity();
+
+                    int  dist = a->GetWidth() / 2;
+                    auto connectedItems = c->GetConnectedItemsAtAnchor( a, aPt, types, dist );
+
+                    if( alg::contains( connectedItems, b ) )
+                        aCollector.Remove( b );
+                }
             },
             true /* prompt user regarding locked items */ );
 
@@ -748,6 +764,13 @@ int EDIT_TOOL::doMoveSelection( TOOL_EVENT aEvent, bool aPickReference )
         return 0;
     }
 
+    auto displayConstraintsMessage =
+            [editFrame]( bool constrained )
+            {
+                editFrame->DisplayConstraintsMsg( constrained ? _( "Constrain to H, V, 45" )
+                                                            : wxT( "" ) );
+            };
+
     std::vector<BOARD_ITEM*> sel_items;     // All the items operated on by the move below
     std::vector<BOARD_ITEM*> orig_items;    // All the original items in the selection
 
@@ -758,7 +781,9 @@ int EDIT_TOOL::doMoveSelection( TOOL_EVENT aEvent, bool aPickReference )
 
         if( boardItem )
         {
-            orig_items.push_back( boardItem );
+            if( !is_hover )
+                orig_items.push_back( boardItem );
+
             sel_items.push_back( boardItem );
         }
 
@@ -785,6 +810,8 @@ int EDIT_TOOL::doMoveSelection( TOOL_EVENT aEvent, bool aPickReference )
     bool hasRedrawn3D    = false;
     bool allowRedraw3D   = editFrame->GetDisplayOptions().m_Live3DRefresh;
 
+    displayConstraintsMessage( lock45 );
+
     // Prime the pump
     m_toolMgr->RunAction( ACTIONS::refreshPreview );
 
@@ -1030,6 +1057,7 @@ int EDIT_TOOL::doMoveSelection( TOOL_EVENT aEvent, bool aPickReference )
         else if( evt->IsAction( &PCB_ACTIONS::toggle45 ) )
         {
             lock45 = !lock45;
+            displayConstraintsMessage( lock45 );
             evt->SetPassEvent( false );
         }
         else
@@ -1046,6 +1074,8 @@ int EDIT_TOOL::doMoveSelection( TOOL_EVENT aEvent, bool aPickReference )
     m_dragging = false;
     editFrame->UndoRedoBlock( false );
 
+    m_toolMgr->GetTool<DRAWING_TOOL>()->UpdateStatusBar();
+
     if( hasRedrawn3D && restore_state )
         editFrame->Update3DView( false, true );
 
@@ -1062,12 +1092,10 @@ int EDIT_TOOL::doMoveSelection( TOOL_EVENT aEvent, bool aPickReference )
     // Remove the dynamic ratsnest from the screen
     m_toolMgr->RunAction( PCB_ACTIONS::hideDynamicRatsnest, true );
 
-    // Unselect all items to update flags
+    // Unselect all items to clear selection flags and then re-select the originally selected
+    // items.
     m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
-
-    // Reselect items if they were already selected and we completed the move
-    if( !is_hover && !restore_state )
-        m_toolMgr->RunAction( PCB_ACTIONS::selectItems, true, &orig_items );
+    m_toolMgr->RunAction( PCB_ACTIONS::selectItems, true, &orig_items );
 
     editFrame->PopTool( tool );
     editFrame->GetCanvas()->SetCurrentCursor( KICURSOR::ARROW );
@@ -1213,8 +1241,8 @@ int EDIT_TOOL::FilletTracks( const TOOL_EVENT& aEvent )
         auto processFilletOp =
                 [&]( bool aStartPoint )
                 {
+                    std::shared_ptr<CONNECTIVITY_DATA> connectivity = board()->GetConnectivity();
                     wxPoint anchor = ( aStartPoint ) ? track->GetStart() : track->GetEnd();
-                    auto connectivity = board()->GetConnectivity();
                     auto itemsOnAnchor = connectivity->GetConnectedItemsAtAnchor( track, anchor,
                                                                                   track_types );
 
@@ -1734,12 +1762,17 @@ int EDIT_TOOL::Flip( const TOOL_EVENT& aEvent )
 
 int EDIT_TOOL::Remove( const TOOL_EVENT& aEvent )
 {
+    PCB_BASE_EDIT_FRAME*  editFrame = getEditFrame<PCB_BASE_EDIT_FRAME>();
+
     if( isRouterActive() )
     {
         m_toolMgr->RunAction( PCB_ACTIONS::routerUndoLastSegment, true );
         return 0;
     }
 
+    std::string tool = aEvent.GetCommandStr().value();
+    editFrame->PushTool( tool );
+
     std::vector<BOARD_ITEM*> lockedItems;
     Activate();
 
@@ -1779,6 +1812,7 @@ int EDIT_TOOL::Remove( const TOOL_EVENT& aEvent )
         {
             wxBell();
             canvas()->Refresh();
+            editFrame->PopTool( tool );
             return 0;
         }
 
@@ -1949,6 +1983,7 @@ int EDIT_TOOL::Remove( const TOOL_EVENT& aEvent )
     else
         m_commit->Push( _( "Delete" ) );
 
+    editFrame->PopTool( tool );
     return 0;
 }
 
@@ -2138,7 +2173,7 @@ int EDIT_TOOL::Duplicate( const TOOL_EVENT& aEvent )
                 break;
 
             default:
-                wxASSERT_MSG( false, wxString::Format( "Unhandled item type %d",
+                wxASSERT_MSG( false, wxString::Format( wxT( "Unhandled item type %d" ),
                                                        orig_item->Type() ) );
                 break;
             }
