@@ -23,13 +23,14 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-#include "edit_tool.h"
+#include <tools/edit_tool.h>
+#include <router/router_tool.h>
 #include <pgm_base.h>
-#include "pcb_actions.h"
-#include "pcb_control.h"
-#include "pcb_picker_tool.h"
-#include "pcb_selection_tool.h"
-#include "board_reannotate_tool.h"
+#include <tools/pcb_actions.h>
+#include <tools/pcb_control.h>
+#include <tools/pcb_picker_tool.h>
+#include <tools/pcb_selection_tool.h>
+#include <tools/board_reannotate_tool.h>
 #include <3d_viewer/eda_3d_viewer_frame.h>
 #include <bitmaps.h>
 #include <board_commit.h>
@@ -201,7 +202,7 @@ void PCB_CONTROL::unfilledZoneCheck()
 
     for( const ZONE* zone : board()->Zones() )
     {
-        if( !zone->IsFilled() )
+        if( !zone->GetIsRuleArea() && !zone->IsFilled() )
         {
             unfilledZones = true;
             break;
@@ -226,10 +227,10 @@ void PCB_CONTROL::unfilledZoneCheck()
 
         wxString msg;
         msg.Printf( _( "Not all zones are filled. Use Edit > Fill All Zones (%s) "
-                      "if you wish to see all fills." ),
+                       "if you wish to see all fills." ),
                     KeyNameFromKeyCode( PCB_ACTIONS::zoneFillAll.GetHotKey() ) );
 
-        infobar->ShowMessageFor( msg, 10000, wxICON_WARNING  );
+        infobar->ShowMessageFor( msg, 5000, wxICON_WARNING  );
     }
 }
 
@@ -994,6 +995,9 @@ int PCB_CONTROL::placeBoardItems( std::vector<BOARD_ITEM*>& aItems, bool aIsNew,
         {
             const_cast<KIID&>( item->m_Uuid ) = KIID();
 
+            // Even though BOARD_COMMIT::Push() will add any new items to the group, we're
+            // going to run PCB_ACTIONS::move first, and the move tool will throw out any
+            // items that aren't in the entered group.
             if( selectionTool->GetEnteredGroup() && !item->GetParentGroup() )
                 selectionTool->GetEnteredGroup()->AddItem( item );
         }
@@ -1193,8 +1197,15 @@ int PCB_CONTROL::Redo( const TOOL_EVENT& aEvent )
 int PCB_CONTROL::UpdateMessagePanel( const TOOL_EVENT& aEvent )
 {
     PCB_SELECTION_TOOL* selTool = m_toolMgr->GetTool<PCB_SELECTION_TOOL>();
+    ROUTER_TOOL*        routerTool = m_toolMgr->GetTool<ROUTER_TOOL>();
     PCB_SELECTION&      selection = selTool->GetSelection();
 
+    if( routerTool && routerTool->RoutingInProgress() )
+    {
+        routerTool->UpdateMessagePanel();
+        return 0;
+    }
+
     if( selection.GetSize() == 1 )
     {
         EDA_ITEM*                   item = selection.Front();
@@ -1227,16 +1238,6 @@ int PCB_CONTROL::UpdateMessagePanel( const TOOL_EVENT& aEvent )
         msg = footprint->GetFPID().GetLibItemName().wx_str();
         msgItems.emplace_back( MSG_PANEL_ITEM( _( "Footprint Name" ), msg ) );
 
-        wxDateTime date( static_cast<time_t>( footprint->GetLastEditTime() ) );
-
-        if( footprint->GetLastEditTime() && date.IsValid() )
-        // Date format: see http://www.cplusplus.com/reference/ctime/strftime
-            msg = date.Format( wxT( "%b %d, %Y" ) ); // Abbreviated_month_name Day, Year
-        else
-            msg = _( "Unknown" );
-
-        msgItems.emplace_back( MSG_PANEL_ITEM( _( "Last Change" ), msg ) );
-
         msg.Printf( wxT( "%zu" ), (size_t) footprint->GetPadCount( DO_NOT_INCLUDE_NPTH ) );
         msgItems.emplace_back( MSG_PANEL_ITEM( _( "Pads" ), msg ) );
 
